/**
  ******************************************************************************
  * @file    Key_scan.h
  * @author  sinomcu.com-AE
  * @brief   Header file of Key_scan.c file.
  *    
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#ifndef __KEY_SCAN_H
#define __KEY_SCAN_H

#include "ms32f0xx.h"

typedef enum{ 
  Bit_RESET = 0,
  Bit_SET
}BitAction;

typedef enum _KEY_STATUS_LIST{    /*按键状态*/
  KEY_NULL     = 0x00,            /*无动作*/ 
  KEY_DEBOUNCE = 0x01,            /*确认状态*/ 
  KEY_UP       = 0x02,            /*按键抬起*/ 
  KEY_DOWN     = 0x04,            /*按键按下*/
  KEY_LONG     = 0x08,            /*长按*/ 
}KEY_STATUS_LIST;

typedef struct {                  /*构造按键初始化类*/ 
  uint32_t GPIO_Mode;             /*初始化按键引脚模式*/    
  uint32_t GPIO_Pull;             /*初始化按键引脚上下拉*/
  uint32_t GPIO_Speed;            /*初始化按键引脚速度*/  
  GPIO_TypeDef* GPIOx;            /*初始化按键端口*/
  uint16_t GPIO_Pin_x;            /*初始化按键引脚号*/ 
}Key_Init;

typedef struct _KEY_COMPONENTS{   /*状态机类*/
  FunctionalState KEY_SHIELD;     /*按键屏蔽0:屏蔽，1:不屏蔽*/
  uint8_t KEY_Debounce_Count;     /*按键消抖计数*/  
  uint8_t KEY_LONG_COUNT;         /*按键长按计数*/
  BitAction KEY_LEVEL;            /*虚拟当前IO电平，按下1，抬起0*/
  BitAction KEY_DOWN_LEVEL;       /*按下时IO实际的电平*/
  KEY_STATUS_LIST KEY_STATUS;     /*按键当前状态*/
  KEY_STATUS_LIST KEY_PRE_EVENT;  /*按键上一状态*/ 
  KEY_STATUS_LIST KEY_CUR_EVENT;  /*按键当前事件*/
  BitAction (*READ_PIN)(Key_Init Key);      /*读IO电平函数*/
}KEY_COMPONENTS;

typedef struct{                   /* 按键类*/
  Key_Init* Key;                  /* 继承初始化父类*/
  KEY_COMPONENTS Status;          /* 继承状态机父类*/
}Key_Config;

typedef enum _KEY_LIST {          /*按键注册表*/ 
  KEY1,                           /*用户添加的按钮名称 */
  KEY2,                           /*用户添加的按钮名称 */
  KEY_NUM,                        /*必须要有的记录按钮数量，必须在最后 */
}KEY_LIST;

#define KEY_KEY_SCAN_TIME   10    /*按键轮询周期单位ms,10ms*/
#define KEY_DEBOUNCE_NUM    6     /*按键消抖计数,例程轮询周期10ms,60ms消抖*/
#define KEY_LONG_DOWN_DELAY 60    /*长按计数器为60次轮询,例程轮询周期10ms,600ms长按*/

void Creat_Key(Key_Init* Init);   /*初始化按钮函数*/ 
void ReadKeyStatus(void);         /*状态机函数*/ 

#endif /* __KEY_SCAN_H */ 

/******************************** END OF FILE *********************************/
