/**
  ******************************************************************************
  * @file     LIN_user.c
  * @author   sinomcu.com-AE
  * @brief    LIN slalve,receive master sand answer data and dispose message,send
  *     back slave answer data.
  *    
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#include "ms32f0xx.h"
#include "user_config.h"
#include "GPIO_CFG.h"
#include "LIN_user.h"

extern uint8_t g_lin_master_buff[];         /*include PID,for send*/
extern uint8_t g_lin_slave1_buff[];         /*include PID,for receive*/
extern uint8_t g_lin_slave2_buff[];
LinFrame g_lin_master_frame = {UNCONDITOPMAL_FRAME,NO_ERROR,SET,RESET,g_lin_master_buff};
LinFrame g_lin_slave_frame[SLAVE_NUM] = {
  {UNCONDITOPMAL_FRAME,NO_ERROR,SET,RESET,g_lin_slave1_buff},
  {UNCONDITOPMAL_FRAME,NO_ERROR,SET,RESET,g_lin_slave2_buff},
};
LinState g_lin_bus_state ={LIN_SEND_ANSWER_ID,BUS_IDEL};

/**
  * @brief initial lin arguments
  * @param  
  * @retval
  */ 
void Lin_Init(void) {
  uint8_t count=0;
  g_lin_master_frame.lin_frame_type =UNCONDITOPMAL_FRAME;
  g_lin_master_frame.lin_frame_error=NO_ERROR;
  g_lin_master_frame.lin_answer_flag =SET;   
  g_lin_master_frame.lin_dispose_requset =RESET;
  g_lin_master_frame.buff =g_lin_master_buff;
  g_lin_master_frame.buff[LIN_DATA_POS+1] =SEND_FIX_DATA;  /*0xAA*/

  for(count=0; count <SLAVE_NUM;count++ ) {
    g_lin_slave_frame[count].lin_frame_type =UNCONDITOPMAL_FRAME;
    g_lin_slave_frame[count].lin_frame_error=NO_ERROR;
    g_lin_slave_frame[count].lin_answer_flag =SET;   
    g_lin_slave_frame[count].lin_dispose_requset =RESET;   
  }
  g_lin_slave_frame[SLAVE1].buff= g_lin_slave1_buff;
  g_lin_slave_frame[SLAVE2].buff= g_lin_slave2_buff;  
}

/**
  * @brief lin master send one frame(interval+sync+pid+8data+checksum)
  * @param master send buff pointer,uint8_t *
  * @retval none
  */
void Send_LinFrame(uint8_t * send_buff){
  uint8_t pid=0 ,send_count=0;

  pid =LIN_CaclPID(LIN_SEND_ANSWER_ID);
  send_buff[LIN_PID_POS] =pid;                        /*write pid to sendbuff*/
  send_buff[LIN_CHECKSUM_POS] =LIN_CaclChecksum(pid,send_buff+LIN_DATA_POS);   /*write checksum to sendbuff*/

  MS32_USART_RequestBreakSending(USART1);             /*send sysnc interval segment 13 bit 0*/
  MS32_USART_TransmitData8(USART1,LIN_SYNC_CHAR);     /*send sysnc segment 0x55*/
  while(MS32_USART_IsActiveFlag_TC(USART1)==RESET);

  for(send_count=0;send_count < LIN_SEND_DATA_LEN ;send_count++) {   /*send PID +8 data +cheksum*/
    MS32_USART_TransmitData8(USART1,g_lin_master_frame.buff[send_count]);
    while(MS32_USART_IsActiveFlag_TC(USART1)==RESET);
  }
}

/**
  * @brief lin master send frame head(interval+sync+pid)
  * @param salve ID,uint8_t
  * @retval none
  */
void Loop_SlaveSendHead(uint8_t slave_ID) {
  uint8_t pid=0;

  pid =LIN_CaclPID(slave_ID);  
  MS32_USART_RequestBreakSending(USART1);             /*send sysnc interval segment 13 bit 0*/
  MS32_USART_TransmitData8(USART1,LIN_SYNC_CHAR);     /*send sysnc segment 0x55*/
  while(MS32_USART_IsActiveFlag_TC(USART1)==RESET) { ;}
  MS32_USART_TransmitData8(USART1,pid);               /*send PID*/
  while(MS32_USART_IsActiveFlag_TC(USART1)==RESET) { ;}
}

/**
  * @brief block mode wait receive slave send answer data 
  * @param uint8_t salve ID
  * @param uint8 slave number
  * @param uint32_t LED Pin
  * @retval none
  */
void Loop_OneSlave(uint8_t slave_id,uint8_t slave_no,uint32_t LED_pin) {
  uint32_t Timeout = 0; 

  Loop_SlaveSendHead(slave_id);
  Timeout =RESPONSE_WAIT_TIME;
  while((Timeout !=0 )&&(g_lin_slave_frame[slave_no].lin_dispose_requset !=SET)) {    
    if(MS32_SYSTICK_IsActiveCounterFlag()) {
      Timeout--;
      if(Timeout ==0) {
        g_lin_slave_frame[slave_no].lin_frame_error = TIME_OUT_ERROR;
      }
    }
  } 

  if(g_lin_slave_frame[slave_no].lin_dispose_requset ==SET) {        /*receive finish*/
    g_lin_slave_frame[slave_no].lin_dispose_requset =RESET;
    if( LIN_CompareChecksum(g_lin_slave_frame[slave_no].buff) ==RESET) {  /*checksum error*/
      g_lin_master_frame.lin_frame_error=CHECKSUM_ERROR;             /*checksum error*/
    } 
    else{                                                            /*salve send back data checksum OK*/
      if(g_lin_slave_frame[slave_no].buff[LIN_DATA_POS] ==LED_ON_DATA) {
        LED_ON(LED_pin);
      }
      else if(g_lin_slave_frame[slave_no].buff[LIN_DATA_POS]==LED_OFF_DATA) {
        LED_OFF(LED_pin);
      }
      else {
        g_lin_slave_frame[slave_no].lin_frame_error=DATA_ERROR;      /*data error*/            
      }        
    } 
  } /*if(g_lin_master_frame.lin_dispose_requset ==SET)*/

  if(g_lin_slave_frame[slave_no].lin_frame_error != NO_ERROR) {
    /*user code for lin frame error*/
    g_lin_slave_frame[slave_no].lin_frame_error = NO_ERROR;
  }
}

/**
  * @brief block mode loop slave 
  * @param none
  * @retval none
  */
void Loop_Slave(void) {
  Loop_OneSlave(LIN_SLAVE1_ID,SLAVE1,LED1_Pin);
  Loop_OneSlave(LIN_SLAVE2_ID,SLAVE2,LED2_Pin); 
}

/**
  * @brief  Function called in USART IT Handler
  * @param  
  * @retval 
  */
void USART_LBD_Callback(void) {
  g_lin_bus_state.bus_state =LIN_SYSNC;
  if( MS32_USART_IsActiveFlag_RXNE(USART1) ) {
    MS32_USART_ReceiveData8(USART1);
  }
  MS32_USART_EnableIT_RXNE(USART1);              /*after Break Field?Revice sysnc segment*/
} 

/**
  * @brief  Function called in USART IT Handler
  * @param  
  * @retval
  * @note receive slave send answer data 
  */
void USART_RXNE_Callback(void) {
  uint8_t recv_char=0,lin_id=0;
  static uint8_t recv_count=0;
  static uint8_t slave_no;
  /*Read Received character. RXNE flag is cleared by reading of RDR register */ 
  recv_char = MS32_USART_ReceiveData8(USART1);

  switch(g_lin_bus_state.bus_state) {
    case LIN_SYSNC:               /*sysnc segment*/
      if(recv_char ==LIN_SYNC_CHAR) {
        g_lin_bus_state.bus_state =LIN_PID;
        recv_count=0;
      }
      break;
    case LIN_PID:                 /*PID segment*/
      lin_id =GET_ID(recv_char);
      g_lin_bus_state.lin_bus_id =lin_id; 
      switch (lin_id) {
        case LIN_SEND_ANSWER_ID:  /*master send*/
          g_lin_bus_state.lin_bus_id =LIN_SEND_ANSWER_ID;
          g_lin_bus_state.bus_state =LIN_WAIT_NEXT;           
          break;
        case LIN_SLAVE1_ID:       /*receive slave1 send answer*/
          slave_no=SLAVE1;
          g_lin_slave_frame[slave_no].lin_answer_flag=RESET;
          g_lin_slave_frame[slave_no].buff[recv_count++] =recv_char;
          g_lin_bus_state.bus_state =LIN_ANSWER; 
          break;  
        case LIN_SLAVE2_ID:       /*receive slave2 send answer*/
          slave_no=SLAVE2;
          g_lin_slave_frame[slave_no].lin_answer_flag=RESET;
          g_lin_slave_frame[slave_no].buff[recv_count++] =recv_char;
          g_lin_bus_state.bus_state =LIN_ANSWER; 
          break; 
        default :
          g_lin_bus_state.bus_state =LIN_WAIT_NEXT;                       
      }
      break;
    case LIN_ANSWER:              /*receive master send answer data*/
      g_lin_slave_frame[slave_no].buff[recv_count++] =recv_char;
      if(recv_count ==LIN_RECV_DATA_LEN) {  /*frame end*/
        g_lin_bus_state.bus_state =LIN_WAIT_NEXT;
        g_lin_slave_frame[slave_no].lin_dispose_requset =SET;
      }
      break;      
    default :
      g_lin_bus_state.bus_state =LIN_WAIT_NEXT;
  }

  if(g_lin_bus_state.bus_state ==LIN_WAIT_NEXT) {/*wait next LBD*/
    MS32_USART_DisableIT_RXNE(USART1);
    g_lin_bus_state.bus_state =BUS_IDEL; 
  }
}

/******************************** END OF FILE *********************************/