/**
  ******************************************************************************
  * @file    LIN_user.h
  * @author  sinomcu.com-AE
  * @brief   Header file of LIN_user.c file.
  * 
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#ifndef __LIN_USER_H
#define __LIN_USER_H

#include "ms32f0xx.h"
#include "LIN.h"

typedef enum {
  BUS_IDEL = 0U,
  LIN_SYSNC,
  LIN_PID,
  LIN_ANSWER,
  LIN_WAIT_NEXT,
}BusState;

typedef struct {
  uint8_t lin_bus_id;
  BusState bus_state;
}LinState;

typedef enum {
  NO_ERROR       =0x00,
  SYSNC_ERROR    =0x01,
  PID_ERROR      =0x02,
  CHECKSUM_ERROR =0x04,
  TIME_OUT_ERROR =0x08,
  DATA_ERROR     =0x10
}LinFrameError;

typedef struct {
  LINFrameType lin_frame_type;
  LinFrameError lin_frame_error;
  FlagStatus lin_answer_flag;  
  FlagStatus lin_dispose_requset;
  uint8_t * buff;
}LinFrame;

typedef enum {     /*从机注册表*/ 
  SLAVE1,          /*添加从机 */
  SLAVE2,          /*添加从机 */
  SLAVE_NUM,       /*必须要有的记录数量，必须在最后 */
}SlaveList;

#define LIN_FRAM_DATA_LEN         8         /*8 data*/ 
#define LIN_RECV_DATA_LEN         LIN_FRAM_DATA_LEN+2      /*PID+ 8 data+ checksum*/
#define LIN_SEND_DATA_LEN         LIN_FRAM_DATA_LEN+2      /*PID+ 8 data+ checksum*/ 

#define LIN_PID_POS               0x00      /*PID position in master send anser data*/
#define LIN_DATA_POS              0x01      /*start position in master send anser data*/
#define LIN_CHECKSUM_POS          LIN_SEND_DATA_LEN-1      /*checksum position in master send anser data*/ 
#define LED_ON_DATA               0x5A      /*slave send anser data LED state value*/
#define LED_OFF_DATA              0xA5
#define SEND_FIX_DATA             0xAA

void Lin_Init(void);
void Send_LinFrame(uint8_t * send_buff);
void Loop_Slave(void);
void USART_LBD_Callback(void);
void USART_RXNE_Callback(void); 

#endif/* __LIN_USER_H */ 

/******************************** END OF FILE *********************************/
