/**
  ******************************************************************************
  * @file    LIN.h
  * @author  sinomcu.com-AE
  * @brief   Header file of LIN.c file.
  * 
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#ifndef __LIN_H
#define __LIN_H

#include "ms32f0xx.h"

/**
LIN Message Frame(LSB first)：FrameHead + Answer data
    FrameHead：Break Field（>=13 bit Dominate Level+ >=1bit Invisibility level inter-frame space)
               +Sync Byte Field(0x55)+inter-byte space
               +PID（6bit ID+2bit check）
                    bit6:P0 = ID0⊕ID1⊕ID2⊕ID4   
                    bit7:P1 = ~(ID3⊕ID4⊕ID ⊕ID1)
    Answer data:Response space +data  +inter-byte space 
               +checksum(stand checksum,check data,used to Diagnostic Frame;enhanced checksum,check data+PID)
*/

/**
Unconditional Frame    :0x00 - 0x3B
Event Triggered Frame  :0x00 - 0x3B
Sporadic Frame         :0x00 - 0x3B
Diagnostic Frame       :0x3C(master request),0x3D(slave answer)
Reserved Frame         :0x3E,0x3F
*/
typedef enum {
  UNCONDITOPMAL_FRAME = 0U,
  EVENT_TRIGGERED_FRAME,
  SPORADIC_FRAME, 
  DIAGNOSTIC_FRAME,
  RESERVED_FRAME
}LINFrameType;

#define LIN_SYNC_CHAR   0x55
#define GET_ID(pid)     (pid & 0x3F)

uint8_t LIN_CaclPID(uint8_t id);
FlagStatus LIN_CheckPID(uint8_t pid);
uint8_t LIN_CaclChecksum(uint8_t pid,uint8_t *buff);
FlagStatus LIN_CompareChecksum(uint8_t *buff);

#endif/* __LIN_H */ 

/******************************** END OF FILE *********************************/
