/**
  ******************************************************************************
  * @file    LIN_user.h
  * @author  sinomcu.com-AE
  * @brief   Header file of LIN_user.c file.
  * 
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#ifndef __LIN_USER_H
#define __LIN_USER_H

#include "ms32f0xx.h"
#include "LIN.h"

typedef enum {
  BUS_IDEL = 0U,
  LIN_SYSNC,
  LIN_PID,
  LIN_ANSWER,
  LIN_WAIT_NEXT,
}LINStatus;

typedef enum {
  NO_ERROR = 0x00,
  SYSNC_ERROR =0x01,
  PID_ERROR =0x20,
  CHECKSUM_ERROR =0x40,
}LINFrameError;

typedef struct {
  LINFrameType lin_frame_type;
  LINFrameError lin_frame_error;
  FlagStatus lin_dispose_requset;
  uint8_t * recv_buff;
  uint8_t * send_buff;
}LIN_Frame;

#define LIN_FRAM_DATA_LEN         8         /*8 data*/ 
#define LIN_RECV_DATA_LEN         LIN_FRAM_DATA_LEN+2      /*PID+ 8 data+ checksum*/
#define LIN_SEND_DATA_LEN         LIN_FRAM_DATA_LEN+1      /*8 data+ checksum*/ 

#define LIN_RECV_DATA_POS         0x01      /*used data position in slave receive anser data */ 
#define LIN_RECV_DATA_ID          LIN_SEND_ANSWER_FID      /*identify dispose or not in slave receive anser data */
#define LIN_RECV_CHECHSUM_POS     LIN_FRAM_DATA_LEN+1      /*checksum position in slave receive anser data */
#define LIN_SEND_DATA_POS         0x00      /*position in slave send anser data */ 
#define LED_ON_SEND_DATA          0x5A
#define LED_OFF_SEND_DATA         0xA5

void LinInit(void);
void LIN_RecvDataProcess(void);
void Upata_SendBuff(void);

#endif/* __LIN_USER_H */ 

/******************************** END OF FILE *********************************/
